/***********************/
/* initCommunication.c */
/***********************/

#include <stdlib.h>
#include <stdio.h>
#include <error.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <unistd.h>

#include "initCommunication.h"
#include "client-serveur.h"

// Fonction utilisee cote client UDP
// Elle cree une socket vers le point d'acces UDP correspondant au couple
// hote HOST / port PORT.
// Renvoie le file descriptor de cette socket (ou bien quitte
// le programme s'il y a eu une erreur).
int creationSocketPointDAcces() {
  //
  // The following code is an adaptation from the example in man getaddrinfo
  //

  int socketPointDAcces;
  struct addrinfo hints;
  struct addrinfo *result, *rp;
  int s;
  int rc;

  // Obtain address(es) matching host/port
  memset(&hints, 0, sizeof(struct addrinfo));
  hints.ai_family = AF_UNSPEC;    /* Allow IPv4 or IPv6 */
  hints.ai_socktype = SOCK_DGRAM; /* Datagram socket */
  hints.ai_flags = 0;
  hints.ai_protocol = 0;          /* Any protocol */
  
  s = getaddrinfo(HOST, PORT, &hints, &result);
  if (s != 0) {
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, 
		  "getaddrinfo on hostname \"%s\": %s\n", HOST, gai_strerror(s));
  }
  
  // getaddrinfo() returns a list of address structures.
  // Try each address until we successfully connect(2).
  // If socket(2) (or connect(2)) fails, we (close the socket
  // and) try the next address. */
  
  for (rp = result; rp != NULL; rp = rp->ai_next) {
    socketPointDAcces = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
    if (socketPointDAcces == -1)
      continue;

    rc = connect(socketPointDAcces , rp->ai_addr, rp->ai_addrlen);
    if (rc >= 0)
      break;                  /* Success */

    if (close(socketPointDAcces) < 0)
      error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "close");
  }

  freeaddrinfo(result);           /* No longer needed */

  if (rp == NULL) {               /* No address succeeded */
    fprintf(stderr, "Could not connect to \"%s\" on port \"%s\" ==> Abort\n", HOST, PORT);
    exit(EXIT_FAILURE);
  }

  return socketPointDAcces;
}

// Fonction envoyee par Pascal Hennequin a Michel Simatic (en effet, la 
// methode proposee en exemple de man getaddrinfo se melange les pedales
// entre IPv4 et IPv6
int bind_any(int port){
  struct sockaddr_in6 serveraddr;
  int sfd;
  
  sfd = socket(AF_INET6, SOCK_DGRAM, IPPROTO_UDP);
  if (sfd  < 0) return(-1);

  memset(&serveraddr, 0, sizeof(serveraddr));
  serveraddr.sin6_family = AF_INET6;
  serveraddr.sin6_port   = htons(port);
  serveraddr.sin6_addr   = in6addr_any; // means any v6 and any v4 the same time

  if (bind(sfd, (struct sockaddr *)&serveraddr, sizeof(serveraddr)) < 0) {
    close(sfd);
    error_at_line(EXIT_FAILURE, errno,__FILE__,__LINE__,"bind");
  }
  
  //  printf("bind OK :  SOCKET=%d, ADDR = ANYv6+v4 =[::], PORT=%d\n",sfd,port);
  return sfd;
}

// Fonction utilisee cote serveur UDP
// Elle cree le point d'acces UDP correspondant au port PORT.
// Renvoie le file descriptor de ce point d'acces (ou bien quitte
// le programme s'il y a eu une erreur).
int creationPointDAcces() {
  return bind_any(atoi(PORT));
}

