/*********************************/
/* serveurUDP_1_tache_n_client.c */
/*********************************/

#include <stdlib.h>
#include <stdio.h>
#include <error.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <unistd.h>

#include "initCommunication.h"
#include "client-serveur.h"

//
// Constantes de fonctionnement du programme
//
//
#define NB_MAX_REPONSES 10 // Nombre maximum de reponses que le serveur fait
                           // a un client (avant de lui repondre REPONSE_KO) 

void gestionRequete(int fdPointDAcces,
		    struct sockaddr_storage addrEmetteur,
		    socklen_t addrEmetteur_len,
		    message_t requete) {
  static int rangClient = 0;
  message_t reponse;
  
  // On determine l'emetteur du message
  char host[NI_MAXHOST], service[NI_MAXSERV];
  int rc = getnameinfo((struct sockaddr *) &addrEmetteur, addrEmetteur_len,
		       host, NI_MAXHOST,
		       service, NI_MAXSERV, NI_NUMERICSERV);
  if (rc != 0)
      error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "getnameinfo");

  // On traite la requete
  if (requete.infoSup.nbReponsesAuClient == 0) {
    // C'est un nouveau client
    rangClient++;
    printf("%6d-ieme client a traiter (%s:%s)\n", rangClient, host, service);
  }
  if (requete.infoSup.nbReponsesAuClient < NB_MAX_REPONSES) {
    reponse.typ = REQUETE_OK;
    CALCUL_PAYLOAD(reponse.infoSup.payload, requete.infoSup.nbReponsesAuClient+1);
    //usleep(100000);
  } else {
    reponse.typ = REQUETE_KO;
  }
  ssize_t nbWrite = sendto(fdPointDAcces, &reponse, sizeof(reponse), 0,
			   (struct sockaddr *) &addrEmetteur, addrEmetteur_len);
  if (nbWrite != sizeof(reponse))
      error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "sendto");
}

void gestionPointDAcces(int fdPointDAcces) {
  do {
    // Attente message
    struct sockaddr_storage addrEmetteur;
    socklen_t addrEmetteur_len = sizeof(addrEmetteur);
    message_t requete;
    
    ssize_t nbRead = recvfrom(fdPointDAcces, &requete, sizeof(requete), 0,
                       (struct sockaddr *) &addrEmetteur, &addrEmetteur_len);
    if (nbRead != sizeof(requete))
      error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "recvfrom");

    // Gestion de cette requete
    gestionRequete(fdPointDAcces, addrEmetteur, addrEmetteur_len, requete);    
  } while (1);
}

int main() {
  int fdPointDAcces = creationPointDAcces();

  gestionPointDAcces(fdPointDAcces);

  return EXIT_SUCCESS;
}
